/* $Id: ARCFilenameCDXRecordFilter.java 11 2007-07-24 10:11:24Z kfc $
 * $Date: 2007-07-24 12:11:24 +0200 (Tue, 24 Jul 2007) $
 * $Revision: 11 $
 * $Author: kfc $
 *
 * The Netarchive Suite - Software to harvest and preserve websites
 * Copyright 2004-2007 Det Kongelige Bibliotek and Statsbiblioteket, Denmark
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

package dk.netarkivet.common.utils.cdx;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import dk.netarkivet.common.exceptions.ArgumentNotValid;

/**
 * A filter to use in CDXReader when finding CDXRecords matching a
 * filename-pattern.
 *
 */

public class ARCFilenameCDXRecordFilter extends SimpleCDXRecordFilter {
    private String arcfilenamepattern;
    private Pattern p;

    public ARCFilenameCDXRecordFilter(String arcfilenamepattern,
            String filtername) throws ArgumentNotValid {
        super(filtername);
        ArgumentNotValid.checkNotNullOrEmpty(arcfilenamepattern,
                "arcfilenamepattern");
        this.arcfilenamepattern = arcfilenamepattern;
        this.p = Pattern.compile(arcfilenamepattern);
    }

    public String getFilenamePattern() {
         return this.arcfilenamepattern;
    }

    public boolean process(CDXRecord cdxrec) {
        Matcher m = p.matcher(cdxrec.getArcfile());
        if (m.matches()) {
            return true;
        }
        return false;
    }
}
