/* File:        $Id: UseTestRemoteFile.java 96 2007-10-16 07:28:08Z kfc $
 * Revision:    $Revision: 96 $
 * Author:      $Author: kfc $
 * Date:        $Date: 2007-10-16 09:28:08 +0200 (Tue, 16 Oct 2007) $
 *
 * The Netarchive Suite - Software to harvest and preserve websites
 * Copyright 2004-2007 Det Kongelige Bibliotek and Statsbiblioteket, Denmark
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

package dk.netarkivet.testutils.preconfigured;

import java.io.File;

import dk.netarkivet.common.Settings;
import dk.netarkivet.common.distribute.TestRemoteFile;
import dk.netarkivet.common.exceptions.ArgumentNotValid;

/**
 * A preconfigure class for using TestRemoteFile instead of FTP
 *
 */

public class UseTestRemoteFile implements TestConfigurationIF {
    private String originalRemoteFileClass;

    public void setUp() {
        originalRemoteFileClass = Settings.get(Settings.REMOTE_FILE_CLASS);
        Settings.set(Settings.REMOTE_FILE_CLASS, TestRemoteFile.class.getName());
        try {
            Settings.create(Settings.HTTPREMOTEFILE_PORT_NUMBER,
                            Integer.toString(5442));
        } catch (ArgumentNotValid e) {
            Settings.set(Settings.HTTPREMOTEFILE_PORT_NUMBER,
                            Integer.toString(5442));
        }
        try {
            Settings.create(Settings.HTTPSREMOTEFILE_KEYSTORE_FILE,
                            new File(dk.netarkivet.common.distribute.TestInfo.ORIGINALS_DIR, "testkeystore").getPath());
        } catch (ArgumentNotValid e) {
            Settings.set(Settings.HTTPSREMOTEFILE_KEYSTORE_FILE,
                         new File(dk.netarkivet.common.distribute.TestInfo.ORIGINALS_DIR, "testkeystore").getPath());
        }
        try {
            Settings.create(Settings.HTTPSREMOTEFILE_KEYSTORE_PASSWORD,
                            "testpass");
        } catch (ArgumentNotValid e) {
            Settings.set(Settings.HTTPSREMOTEFILE_KEYSTORE_PASSWORD,
                            "testpass");
        }

        try {
            Settings.create(Settings.HTTPSREMOTEFILE_KEY_PASSWORD,
                            "testpass2");
        } catch (ArgumentNotValid e) {
            Settings.set(Settings.HTTPSREMOTEFILE_KEY_PASSWORD,
                            "testpass2");
        }
    }

    public void tearDown() {
        TestRemoteFile.removeRemainingFiles();
        Settings.set(Settings.REMOTE_FILE_CLASS, originalRemoteFileClass);
    }
}
